//---------------------------------------------------------------------------------------
// Автор: Редькин Павел 
// Дата: 03.10.2017
//
// Частототомер-периодометр с символьным ЖКИ 1602ZFA
// 
// fx_in - входная  измеряемая частота  1 Гц ...250 МГц 
//         входной измеряемый период  1 мкс ...100 с  
//  
// clk - глобальный синхросигнал 50 МГц
//
// Платоформа -  Cyclone-IVE     EP4CE6E22C8 
//---------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------
//  clk     PIN_23
//  nrst    PIN_125   // Reset
//---------------------------------------
//   Светодиоды
//  led[3]  PIN_11
//  led[2]  PIN_10
//  led[1]  PIN_7
//  led[0]  PIN_3
//---------------------------------------
//    Кнопки 
//  key[0]  PIN_136  // Pull_UP //  (key_up)  
//  key[1]  PIN_141  // Pull_UP //  (key_down)
//  key[2]  PIN_138  // Pull_UP //  (key_left)
//  key[3]  PIN_137  // Pull_UP //  (key_right)
//  key[4]  PIN_142  // Pull_UP //  (key_z)
//---------------------------------------
//   Звук 
//  busser  PIN_1
//---------------------------------------
//  Интерфейс LCD 
//  LCD_N   PIN_115
//  LCD_P   PIN_114
//  dat[7]  PIN_113 
//  dat[6]  PIN_112
//  dat[5]  PIN_111
//  dat[4]  PIN_110
//  dat[3]  PIN_106
//  dat[2]  PIN_105 
//  dat[1]  PIN_104
//  dat[0]  PIN_103
//  e       PIN_120
//  rs      PIN_98
//  rw      PIN_100 
//---------------------------------------
// Входная измеряемая частота
//  fx_in   PIN_28  
//
// Тестовые  генераторы:   
//  generator_250_MHz       PIN_39  
//  generator_125_MHz       PIN_31  
//  generator_1_MHz         PIN_32 
//  generator_10_kHz        PIN_33 
//  generator_1_kHz         PIN_34   
//  generator_100_Hz        PIN_38  
//  generator_1_Hz          PIN_30
//  generator_05_Hz         PIN_42 
//  generator_025_Hz        PIN_43  
//  generator_0125_Hz       PIN_44 
//  generator_00125_Hz      PIN_46 
//--------------------------------------------------------------------------
module Fmeter_3  (
                     input nrst, 
							input clk,
							input fx_in, 
							input [4:0] key, 
							output [7:0] dat, 
							output e, 
							output rs, 
							output rw, 
							output LCD_N, 
							output LCD_P, 
							output [3:0] led, 
							output buzzer,
							output generator_250_MHz,
							output generator_125_MHz,
							output generator_1_MHz,    
							output generator_10_kHz,
							output generator_1_kHz,
							output generator_100_Hz,
							output generator_1_Hz,
							output generator_05_Hz,
							output generator_025_Hz,
							output generator_0125_Hz, 
							output generator_00125_Hz
  		              );
//--------------------------------------------------------------------------

wire [7:0] edge_noise_key; // линии  кнопок управления с подавленным дребезгом и  
                            // длительностью 1 период внутреннего глобального синхросигнала  
									 
wire [7:0]out_noise_key; // линии  кнопок управления с подавленным дребезгом  						 
																		 
wire [7:0] en_buzzer_button;  // линии разрешения звука от кнопок   									   						  						
											 
wire clk_pll; // выходной сигнал PLL1  - 250 МГц												
   
// Регистры светодиодов																	 
reg[3:0] led_reg = 4'b1111;  // регистр светодиодов  
 
// Регистры кнопок и звука от кнопок 
reg clk_1_kHz = 0;  // синхросигнал 1 кГц 
reg[15:0] counter_2_kHz = 0; //  счетчик  для генерации частоты 2000 Гц
reg [7:0] en_buzzer_key = 0;     // разрешение  звука от кнопок

// Регистры делителей ччастоты 
reg clk_1_MHz = 0;  // синхросигнал 1 МГц
reg [15:0] counter_2_MHz = 0; //  счетчик  для генерации частоты 2 МГц
reg [15:0] counter_20_kHz = 0; //  счетчик  для генерации частоты 20 кГц
reg [31:0] counter_200_Hz = 0; //  счетчик  для генерации частоты 200 Гц
reg [31:0] counter_2_Hz = 0; //  счетчик  для генерации частоты 2 Гц
reg [31:0] counter_0025_Hz = 0; //  счетчик  для генерации частоты 0,025 МГц

// Регистры LCD 
reg [25:0] counter_s = 0;  // счетчик для синхронизации   индикации на LCD
reg en = 0;   // управление стробом  данных-команд LCD  
reg Init_LCD_start = 0; // флаг разрешения начальной инициализации LCD  
reg Funk_LCD_start = 0; // флаг разрешения текущего функционирования LCD
reg [7:0]  data = 0; // переменная слова данных LCD 
reg rsn = 0; // переменная признака  данные-команды LCD  

// Регистры управления
reg mode = 0; // регистр режима: 0 -  измерение, 1 калибровка  

// Регистры периодометра
reg [31:0] counter_period = 0; // счетчик отсчета периода входной измеряемой частоты
reg [31:0] data_period = 0; // регистр хранения периода входной измеряемой частоты

reg Q0 = 0; // регистры  трехступенчатого триггера формирования рабочих сигналов
reg Q1 = 0;
reg Q2 = 0;
wire clear_fx_in; // короткий импульс по по фронту входной измеряемой частоты

reg perep = 0; // флаг превышения предела отображения на ЖКИ измеренного периода
              // при установленном флаге измеренный период  не отображается  

// счетчики импульсов счетной частоты   
reg[3:0] count_px_in_ed = 0;  // счетчик счета  единиц импульсов счетной частоты 
reg[3:0] count_px_in_des = 0;  // счетчик счета  десятков импульсов  счетной частоты 
reg[3:0] count_px_in_sot = 0;  // счетчик счета  сотен импульсов счетной  частоты 
reg[3:0] count_px_in_tis = 0;  // счетчик счета  тысяч импульсов счетной частоты 
reg[3:0] count_px_in_dtis = 0;  // счетчик счета  десятков импульсов счетной частоты 
reg[3:0] count_px_in_stis = 0;  // счетчик счета сотен тысяч импульсов счетной частоты 
reg[3:0] count_px_in_mill = 0;  // счетчик счета  миллионов импульсов счетной частоты 
reg[3:0] count_px_in_dmill = 0;  // счетчик счета десятков миллионов импульсов счетной частоты 
reg[3:0] count_px_in_smill = 0;  // счетчик счета сотен миллионов импульсов счетной частоты 

// регистры хранения измеренного значения периода входной измеряемой частоты в десятичном виде  
reg[3:0] data_px_in_ed = 0;  // регистр хранения единиц импульсов входного измеряемого периода 
reg[3:0] data_px_in_des = 0;  // регистр хранения  десятков импульсов входного измеряемого периода 
reg[3:0] data_px_in_sot = 0;  // регистр хранения  сотен импульсов входного измеряемого периода 
reg[3:0] data_px_in_tis = 0;  // регистр хранения   тысяч импульсов входного измеряемого периода
reg[3:0] data_px_in_dtis = 0;  // регистр хранения  десятков импульсов входного измеряемого периода 
reg[3:0] data_px_in_stis = 0;  // регистр хранения  сотен тысяч импульсов входного измеряемого периода 
reg[3:0] data_px_in_mill = 0;  // регистр хранения   миллионов импульсов входного измеряемого периода
reg[3:0] data_px_in_dmill = 0;  // регистр хранения  десятков миллионов импульсов входного измеряемого периода 
reg[3:0] data_px_in_smill = 0;  // регистр хранения  сотен миллионов импульсов входного измеряемого периода 
 
// Регистры частотомера
reg[25:0] counter_time = 0;  //  счетчик  отсчета  метки времени строба 1 с

// счетчики импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_ed = 0;  // счетчик счета  единиц импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_des = 0;  // счетчик счета  десятков импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_sot = 0;  // счетчик счета  сотен импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_tis = 0;  // счетчик счета  тысяч импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_dtis = 0;  // счетчик счета  десятков импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_stis = 0;  // счетчик счета сотен тысяч импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_mill = 0;  // счетчик счета  миллионов импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_dmill = 0;  // счетчик счета десятков миллионов импульсов входной измеряемой частоты 
reg[3:0] count_fx_in_smill = 0;  // счетчик счета сотен миллионов импульсов входной измеряемой частоты 

// регистры хранения измеренного значения входной измеряемой частоты в десятичном виде 
reg[3:0] data_fx_in_ed = 0;  // регистр хранения единиц импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_des = 0;  // регистр хранения  десятков импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_sot = 0;  // регистр хранения  сотен импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_tis = 0;  // регистр хранения   тысяч импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_dtis = 0;  // регистр хранения  десятков импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_stis = 0;  // регистр хранения  сотен тысяч импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_mill = 0;  // регистр хранения   миллионов импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_dmill = 0;  // регистр хранения  десятков миллионов импульсов входной измеряемой частоты 
reg[3:0] data_fx_in_smill = 0;  // регистр хранения  сотен милллионов импульсов входной измеряемой частоты 

// Регистры генераторов 
reg out_125_MHz = 0;

reg out_10_kHz = 0;
reg out_100_Hz = 0;

reg out_1_Hz = 0;
reg out_05_Hz = 0;
reg out_025_Hz = 0;
reg out_0125_Hz = 0;
reg out_00125_Hz = 0;

// Линии сигналов измерения частоты
reg strob = 0; //  строб 1 с
reg strob_count = 0; // вспомогательный строб  
reg strob_fx_in; // захваченнная стробом пачка импульсов входной измеряемой частоты
 
reg [25:0] time_strob_reg = 0; // регистр делителя для получения метки времени строба 1 с

// Константные параметры  
parameter   GLOBAL_CLOCK  = 50_000_000;  // частота глобального синхросигнала, Гц

parameter  DEL_2_MHz = 24;  // делитель  для получения частоты 2 МГц из 50_000_000 Гц
parameter  DEL_20_kHz =  2_499;  // делитель  для получения частоты 20 кГц из 50_000_000 Гц
parameter  DEL_2_kHz = 24_999;  // делитель  для получения частоты 2 кГц из 50_000_000 Гц
parameter  DEL_200_Hz = 249_999;  // делитель  для получения частоты 200 Гц из 50_000_000 Гц
parameter  DEL_2_Hz = 24_999_999;  // делитель  для получения частоты 2 Гц из 50_000_000 Гц 
parameter  DEL_0025_Hz = 1_999_999_999;  // делитель  для получения частоты 0,025 Гц из 50_000_000 Гц 

parameter  TIME_STROB  = 50_000_000;   // делитель для получения метки времени строба 1 с 
parameter  DELTA_STROB = 1;  // элементарное значение изменения длительности основного строба при калибровке

//версия проекта ПЛИС
parameter  ver_H = "3";
parameter  ver_L = "1";

parameter   MAX_DEC = 9; // предел десятичного счетчика 
parameter   MODE_0 = 0; // режим измерений   

parameter   PREV = 4'hF; // символ указания на превышение предела индикации  

// константы задания временных интервалов управления LCD  
//parameter   VALUE_1 = 50; // для симуляции   
parameter   VALUE_1 = 50_000; // рабочее значение 
parameter   VALUE_2 = 2 * VALUE_1; //
parameter   VALUE_3 = 3 * VALUE_1; //
parameter   VALUE_4 = 4 * VALUE_1; //
parameter   VALUE_5 = 5 * VALUE_1; //
parameter   VALUE_6 = 6 * VALUE_1; //
parameter   VALUE_7 = 7 * VALUE_1; //
parameter   VALUE_8 = 8 * VALUE_1; //
parameter   VALUE_9 = 9 * VALUE_1; //
parameter   VALUE_10 = 10 * VALUE_1; //
parameter   VALUE_11 = 11 * VALUE_1; // 
parameter   VALUE_12 = 12 * VALUE_1; //
parameter   VALUE_13 = 13 * VALUE_1; //
parameter   VALUE_14 = 14 * VALUE_1; //
parameter   VALUE_15 = 15 * VALUE_1; // 
parameter   VALUE_16 = 16 * VALUE_1; //
parameter   VALUE_17 = 17 * VALUE_1; // 
parameter   VALUE_18 = 18 * VALUE_1; //
parameter   VALUE_19 = 19 * VALUE_1; // 
parameter   VALUE_20 = 20 * VALUE_1; // 
parameter   VALUE_21 = 21 * VALUE_1; //
parameter   VALUE_22 = 22 * VALUE_1; // 
parameter   VALUE_23 = 23 * VALUE_1; // 
parameter   VALUE_24 = 24 * VALUE_1; // 
parameter   VALUE_25 = 25 * VALUE_1; // 
parameter   VALUE_26 = 26 * VALUE_1; // 
parameter   VALUE_27 = 27 * VALUE_1; // 
parameter   VALUE_28 = 28 * VALUE_1; //
parameter   VALUE_29 = 29 * VALUE_1; //
parameter   VALUE_30 = 30 * VALUE_1; //
parameter   VALUE_31 = 31 * VALUE_1; //
parameter   VALUE_32 = 32 * VALUE_1; //
parameter   VALUE_33 = 33 * VALUE_1; //
parameter   VALUE_34 = 34 * VALUE_1; //
parameter   VALUE_35 = 35 * VALUE_1; //
parameter   VALUE_36 = 36 * VALUE_1; //
parameter   VALUE_37 = 37 * VALUE_1; //
parameter   VALUE_38 = 38 * VALUE_1; //
parameter   VALUE_39 = 39 * VALUE_1; //
parameter   VALUE_40 = 40 * VALUE_1; //
parameter   VALUE_41 = 41 * VALUE_1; //
parameter   VALUE_42 = 42 * VALUE_1; //
parameter   VALUE_43 = 43 * VALUE_1; //
parameter   VALUE_44 = 44 * VALUE_1; //
parameter   VALUE_45 = 45 * VALUE_1; //
parameter   VALUE_46 = 46 * VALUE_1; //
parameter   VALUE_47 = 47 * VALUE_1; //
parameter   VALUE_48 = 48 * VALUE_1; //
parameter   VALUE_49 = 49 * VALUE_1; //
parameter   VALUE_50 = 50 * VALUE_1; //
parameter   VALUE_51 = 51 * VALUE_1; //
parameter   VALUE_52 = 52 * VALUE_1; //
parameter   VALUE_53 = 53 * VALUE_1; //
parameter   VALUE_54 = 54 * VALUE_1; //
parameter   VALUE_55 = 55 * VALUE_1; //
parameter   VALUE_56 = 56 * VALUE_1; //
parameter   VALUE_57 = 57 * VALUE_1; //
parameter   VALUE_58 = 58 * VALUE_1; //
parameter   VALUE_59 = 59 * VALUE_1; //
parameter   VALUE_60 = 60 * VALUE_1; //
parameter   VALUE_61 = 61 * VALUE_1; //
parameter   VALUE_62 = 62 * VALUE_1; //
parameter   VALUE_63 = 63 * VALUE_1; //
parameter   VALUE_64 = 64 * VALUE_1; //
parameter   VALUE_65 = 65 * VALUE_1; //
parameter   VALUE_66 = 66 * VALUE_1; //
parameter   VALUE_67 = 67 * VALUE_1; //
parameter   VALUE_68 = 68 * VALUE_1; //
parameter   VALUE_69 = 69 * VALUE_1; //
parameter   VALUE_70 = 70 * VALUE_1; //
parameter   VALUE_71 = 71 * VALUE_1; //
parameter   VALUE_72 = 72 * VALUE_1; //
parameter   VALUE_73 = 73 * VALUE_1; //
parameter   VALUE_74 = 74 * VALUE_1; //
parameter   VALUE_75 = 75 * VALUE_1; //
parameter   VALUE_76 = 76 * VALUE_1; //
parameter   VALUE_77 = 77 * VALUE_1; //
parameter   VALUE_78 = 78 * VALUE_1; //
parameter   VALUE_79 = 79 * VALUE_1; //
parameter   VALUE_80 = 80 * VALUE_1; //
parameter   VALUE_81 = 81 * VALUE_1; //
parameter   VALUE_82 = 82 * VALUE_1; //
parameter   VALUE_83 = 83 * VALUE_1; //
parameter   VALUE_84 = 84 * VALUE_1; //
parameter   VALUE_85 = 85 * VALUE_1; //
parameter   VALUE_86 = 86 * VALUE_1; //
parameter   VALUE_87 = 87 * VALUE_1; //
parameter   VALUE_88 = 88 * VALUE_1; //
parameter   VALUE_89 = 89 * VALUE_1; //
parameter   VALUE_90 = 90 * VALUE_1; //
parameter   VALUE_91 = 91 * VALUE_1; //
parameter   VALUE_92 = 92 * VALUE_1; //
parameter   VALUE_93 = 93 * VALUE_1; //
parameter   VALUE_94 = 94 * VALUE_1; //
parameter   VALUE_95 = 95 * VALUE_1; //
parameter   VALUE_96 = 96 * VALUE_1; //

parameter   VALUE_300 = 300 * VALUE_1; //  
parameter   VALUE_1000 = 1000 * VALUE_1; // 
 
// константы начальной инициализации LCD  
parameter  Init_LCD_2s_8bit_5x7 = 8'b0011_1100; 
parameter  Init_LCD_d_ON_k_OFF  = 8'b0000_1100;
parameter  Init_LCD_d_NS_k_S    = 8'b0000_0110;
parameter  Init_LCD_d_start     = 8'b0000_0001;
parameter  Init_LCD_1_start     = 8'b0000_0010;
parameter  Init_LCD_2_start     = 8'b1100_0000;


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   Измерение периода методом подсчета импульсов счетной частоты (1 МГц) за измеряемый период  
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

// трехступенчатый триггер для формирования положительного импульса  длительностью 1 период clk   	
// по фронту входной измеряемой частоты 
// первая ступень - простая задержка на один период  clk
// вторая и третья ступени с комбинационной схемой формируют положительный импульс   
always@(posedge clk) 	  
  begin  
   Q0 <= fx_in;
   Q1 <= Q0;   
   Q2 <= Q1;
  end  
	  
// комбинационная часть формирователя положительного импульса длительностью 1 период clk
// по фронту входной измеряемой частоты   	
assign clear_fx_in = Q1 &(~Q2);	


// непрерывный счет счетных импульсов  1 МГц  для измерения периода входной измеряемой частоты
// и обнуление счетчика в начале каждого периода входной измеряемой частоты 
always @(posedge clk_1_MHz or posedge clear_fx_in) 
begin
   if (clear_fx_in)
	  begin 
	    // обнуление счетчиков импульсов счетной частоты 1 МГц 
		 // коротким импульсом в начале каждого периода входной измеряемой частоты
       count_px_in_ed <= 0; 
       count_px_in_des <= 0; 
       count_px_in_sot <= 0; 
       count_px_in_tis <= 0; 
       count_px_in_dtis <= 0; 
       count_px_in_stis <= 0; 
       count_px_in_mill <= 0; 
       count_px_in_dmill <= 0; 
       count_px_in_smill <= 0; 		     
     end  
   else
     begin
	  // счет импульсов счетной частоты 1 МГц,
	   // захваченных периодом входной измеряемой частоты  
	  count_px_in_ed <= count_px_in_ed + 1; 
	  if (count_px_in_ed == MAX_DEC)
	     begin
		     count_px_in_ed <= 0;
			  count_px_in_des <= count_px_in_des + 1; 
               if (count_px_in_des == MAX_DEC)
                   begin 
					count_px_in_des <= 0;  
				       count_px_in_sot <= count_px_in_sot + 1; 
				     if (count_px_in_sot == MAX_DEC) 
					     begin
						     count_px_in_sot <= 0; 
							  count_px_in_tis <= count_px_in_tis + 1; 
							  if (count_px_in_tis == MAX_DEC)
							     begin
								     count_px_in_tis <= 0; 
									  count_px_in_dtis <= count_px_in_dtis + 1; 
								     if (count_px_in_dtis == MAX_DEC)
									     begin
									        count_px_in_dtis <= 0;  
						                        count_px_in_stis <= count_px_in_stis + 1;
			                                      if (count_px_in_stis == MAX_DEC) 
											     begin
												     count_px_in_stis <= 0;
													  count_px_in_mill <= count_px_in_mill + 1; 
													  if (count_px_in_mill == MAX_DEC) 
													     begin
														     count_px_in_mill <= 0; 
															  count_px_in_dmill <= count_px_in_dmill + 1; 
															  if (count_px_in_dmill == MAX_DEC)
						               					           begin
								               				          count_px_in_dmill <= 0; 
															          count_px_in_smill <= count_px_in_smill + 1;	 
																  end 
														  end 
			                             end 
									     end 
								end  
				        end 		
			     end
		  end
	  //-----------------------------------------------------------------------------------
	  end 
end 	  


// управление флагом превышения предела отображения на ЖКИ измеренного периода
// При установленном флаге измеренный период  не отображается    
always@(posedge clk_1_MHz)
begin
	  if (data_fx_in_smill == 0)
	     begin 
		    if  (data_fx_in_dmill == 0)
		      begin 
		        if (data_fx_in_mill <= 1)	  
		           begin
		             perep <= 0;
			        end
				  else 
	              begin 
		             perep <= 1;
                 end  			
			   end
			 else  	
	         begin 
		        perep <= 1;
            end          				
        end  				
     else 
	     begin 
		    perep <= 1;
        end          	  
end 
	  

// захват счетчиков в регистры в начале каждого периода входной измеряемой частоты
// период входной измеряемой частоты синхронизирован с глобальным синхросигналом      
always @(posedge fx_in or posedge (!nrst))  
begin
 if (!nrst)   
     begin
	    // обнулить регистры захвата от кнопки сброса    
       data_px_in_ed <= 0; 
       data_px_in_des <= 0; 
       data_px_in_sot <= 0; 
       data_px_in_tis <= 0; 
       data_px_in_dtis <= 0; 
       data_px_in_stis <= 0; 
       data_px_in_mill <= 0; 
       data_px_in_dmill <= 0; 
       data_px_in_smill <= 0; 	 
     end
 else 
     begin 
       if (perep == 0)
		   begin     
		  	  //  захват подсчитанного значения из счетчиков в регистры
		     data_px_in_ed <= count_px_in_ed;
		     data_px_in_des <= count_px_in_des;
           data_px_in_sot <= count_px_in_sot;
		     data_px_in_tis <= count_px_in_tis;
		     data_px_in_dtis <= count_px_in_dtis;
		     data_px_in_stis <= count_px_in_stis;
		     data_px_in_mill <= count_px_in_mill;
		     data_px_in_dmill <= count_px_in_dmill;
			  data_px_in_smill <= count_px_in_smill;
			end 
		else
         begin
	        // индицировать превышение предела отображения на ЖКИ измеренного периода     
           data_px_in_ed <= PREV; 
           data_px_in_des <= PREV; 
           data_px_in_sot <= PREV; 
           data_px_in_tis <= PREV; 
           data_px_in_dtis <= PREV; 
           data_px_in_stis <= PREV; 
           data_px_in_mill <= PREV; 
           data_px_in_dmill <= PREV;
			  data_px_in_smill <= PREV; 
         end          
	 end 	
end 

// получение  синхросигнала 1 МГц  из частоты 50_000_000 Гц
// для измерения периода входного сигнала 
always @(posedge  clk)
begin
   //------------------------------------
	if (counter_2_MHz == DEL_2_MHz)
	  begin
	     counter_2_MHz <= 0;
		  clk_1_MHz <= ~clk_1_MHz;  
	  end
   else
     begin
        counter_2_MHz <= counter_2_MHz + 1;  	 
	  end
	//------------------------------------- 	 
end 


// получение  синхросигнала 10 кГц  из частоты 50_000_000 Гц
// для измерения периода входного сигнала 
always @(posedge  clk)
begin
   //------------------------------------
	if (counter_20_kHz == DEL_20_kHz)
	  begin
	     counter_20_kHz <= 0;
        out_10_kHz <= ~out_10_kHz;  
	  end
   else
     begin
        counter_20_kHz <= counter_20_kHz + 1;  	 
	  end
	//------------------------------------- 	 
end 


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   Измерение частоты методом подсчета импульсов счетной частоты за фиксированный интервал времени (1 с)
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

// получение временных меток - стробов длительностью 1 с
always @(posedge  clk)
begin
 if (!nrst)   
     begin 	  	 
	   // сбросить основной и вспомогательный стробы
	   strob <= 0;
		strob_count <= 0; 
		// обнулить счетчик формирования стробов  
      counter_time <= 0; 
		// обнулить регистры захвата   
      data_fx_in_ed <= 0; 
      data_fx_in_des <= 0; 
      data_fx_in_sot <= 0; 
      data_fx_in_tis <= 0; 
      data_fx_in_dtis <= 0; 
      data_fx_in_stis <= 0; 
      data_fx_in_mill <= 0; 
      data_fx_in_dmill <= 0; 
      data_fx_in_smill <= 0; 		 		
	  end
  
	  counter_time <= counter_time + 1; // накопление  "тела"  строба
     case(counter_time)  
		                      1: begin strob <= 1;  strob_count <= 1; end  // установить основной и вспомогательный стробы
         (time_strob_reg + 1): begin //  захват подсчитанного значения из счетчиков в регистры
			                             data_fx_in_ed <= count_fx_in_ed;
												  data_fx_in_des <= count_fx_in_des;
												  data_fx_in_sot <= count_fx_in_sot;
												  data_fx_in_tis <= count_fx_in_tis;
												  data_fx_in_dtis <= count_fx_in_dtis;
												  data_fx_in_stis <= count_fx_in_stis;
												  data_fx_in_mill <= count_fx_in_mill;
												  data_fx_in_dmill <= count_fx_in_dmill;
												  data_fx_in_smill <= count_fx_in_smill;
	                		              strob <= 0;  //  обнулить основной строб  
										 end  
		   (time_strob_reg + 5): begin  strob_count <= 0; end  // обнулить вспомогательный строб
		  (time_strob_reg + 10): begin counter_time <= 0;  end   // закончить накопление  "тела"  строба           
	  endcase  
end 


// формирование счетной пачки импульсов  входной измеряемой частоты 
// в зависимости от значения переменной mode в счетную пачку захватываются импульсы входной измеряемой частоты 
// или импульсы синхросигнала, или ничего (нулевое содержимое) 
always @(*)
begin 
   case (mode)
           0: strob_fx_in <= strob & fx_in; 
           1: strob_fx_in <= strob & clk;
   endcase
end 

	  
//assign strob_fx_in = strob & ((fx_in & ~mode)|(clk & mode)); 


// подсчет импульсов входной измеряемой частоты, пришедших за время основного строба
always @(posedge strob_fx_in or negedge strob_count)
begin
 if(!strob_count)  
     begin
	    // обнуление счетчиков импульсов входной измеряемой частоты, 
		 // когда вспомогательный строб в низком уровне
       count_fx_in_ed <= 0; 
       count_fx_in_des <= 0; 
       count_fx_in_sot <= 0; 
       count_fx_in_tis <= 0; 
       count_fx_in_dtis <= 0; 
       count_fx_in_stis <= 0; 
       count_fx_in_mill <= 0; 
       count_fx_in_dmill <= 0; 
       count_fx_in_smill <= 0; 		 
     end                  
	  
 else	  
     begin 
	  // счет импульсов входной измеряемой частоты,
	   // захваченных основным стробом  
	  count_fx_in_ed <= count_fx_in_ed + 1; 
	  if (count_fx_in_ed == MAX_DEC)
	     begin
		     count_fx_in_ed <= 0;
			  count_fx_in_des <= count_fx_in_des + 1; 
           if (count_fx_in_des == MAX_DEC)
                       begin 
					  count_fx_in_des <= 0;  
				     count_fx_in_sot <= count_fx_in_sot + 1; 
				   if (count_fx_in_sot == MAX_DEC) 
					     begin
						     count_fx_in_sot <= 0; 
							  count_fx_in_tis <= count_fx_in_tis + 1; 
							  if (count_fx_in_tis == MAX_DEC)
							     begin
								     count_fx_in_tis <= 0; 
									  count_fx_in_dtis <= count_fx_in_dtis + 1; 
								     if (count_fx_in_dtis == MAX_DEC)
									     begin
									        count_fx_in_dtis <= 0;  
						                 count_fx_in_stis <= count_fx_in_stis + 1;
			                          if (count_fx_in_stis == MAX_DEC) 
											     begin
												     count_fx_in_stis <= 0;
													  count_fx_in_mill <= count_fx_in_mill + 1; 
													  if (count_fx_in_mill == MAX_DEC) 
													     begin
														     count_fx_in_mill <= 0; 
															  count_fx_in_dmill <= count_fx_in_dmill + 1; 
															  if (count_fx_in_dmill == MAX_DEC)
						               					  begin
								               				  count_fx_in_dmill <= 0; 
																	  count_fx_in_smill <= count_fx_in_smill + 1;	 
																  end 
														  end 
			                             end 
									     end 
							    end  
				        end 		
			     end
		  end
	 //---------------------------------------------------------------------- 
	 end
//-------------------------------------------------------------------------------	  
end 

// получение  синхросигнала 1 кГц  из частоты 50_000_000 Гц
// для формирования звуковых пачек 
always @(posedge  clk)
begin
   //------------------------------------
	if (counter_2_kHz == DEL_2_kHz)
	  begin
	     counter_2_kHz <= 0;
        clk_1_kHz <= ~clk_1_kHz; // заполнение для формирования звуковых пачек 
	  end
   else
     begin
        counter_2_kHz <= counter_2_kHz + 1;  	 
	  end
	//------------------------------------- 	 
end 


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   Контроллер ЖКИ 
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

// начальная  инициализация и  циклический вывод  данных на LCD
always @(posedge clk)      
begin 
   if (!nrst)   
       begin 	  	 
	     Init_LCD_start <= 1;  // разрешить начальную  инициализацию LCD 
		  Funk_LCD_start <= 0;  // запретить текущее функционирование LCD 
		  counter_s <= 0;     
  	     //  все всетодиоды погасить, один зажечь 
        led_reg[0] <= 0;		  
        led_reg[1] <= 1;
		  led_reg[2] <= 1;
		  led_reg[3] <= 1;
       end
 
   else  if (Init_LCD_start == 1) // если  начальная инициализация LCD разрешена
     begin    
      counter_s <= counter_s + 1;
      //  начальная инициализация LCD 
      case(counter_s) 
        VALUE_1:  begin rsn <= 0; data <= Init_LCD_2s_8bit_5x7;  end  
        VALUE_2:  begin en <= 1;  end  // начало импульса en  
        VALUE_3:  begin en <= 0;  end  // конец импульса en

        VALUE_4:  begin rsn <= 0; data <= Init_LCD_d_ON_k_OFF;  end  
        VALUE_5:  begin en <= 1;  end  // начало импульса en
        VALUE_6:  begin en <= 0;  end  // конец импульса en 
 
        VALUE_7:  begin rsn <= 0; data <= Init_LCD_d_NS_k_S;  end  
        VALUE_8:  begin en <= 1;  end  // начало импульса en
        VALUE_9:  begin en <= 0;  end	// конец импульса en  
	  
        VALUE_10:  begin rsn <= 0; data <= Init_LCD_d_start;  end
        VALUE_11:  begin en <= 1;  end  // начало импульса en
        VALUE_12:  begin en <= 0;  end	  

		  // вывод начальной заставки  на LCD  
        VALUE_13:  begin rsn <= 1; data <= "F";   end  
        VALUE_14:  begin en <= 1;  end  // начало импульса en
        VALUE_15:  begin en <= 0;  end	 // конец импульса en 	  

        VALUE_16:  begin rsn <= 1; data <= "M";   end  
        VALUE_17:  begin en <= 1;  end  // начало импульса en
        VALUE_18:  begin en <= 0;  end	 // конец импульса en	  
	  
        VALUE_19:  begin rsn <= 1; data <= "E";   end  
        VALUE_20:  begin en <= 1;  end  // начало импульса en
        VALUE_21:  begin en <= 0;  end	 // конец импульса en
		  
        VALUE_22:  begin rsn <= 1; data <= "T";   end  
        VALUE_23:  begin en <= 1;  end  // начало импульса en
        VALUE_24:  begin en <= 0;  end	 // конец импульса en
			 
        VALUE_25:  begin rsn <= 1; data <= "E";   end  
        VALUE_26:  begin en <= 1;  end  // начало импульса en
        VALUE_27:  begin en <= 0;  end	 // конец импульса en
		  
        VALUE_28:  begin rsn <= 1; data <= "R";   end  
        VALUE_29:  begin en <= 1;  end  // начало импульса en
        VALUE_30:  begin en <= 0;  end	 // конец импульса en	
	
        VALUE_31:  begin rsn <= 1; data <= " ";   end  
        VALUE_32:  begin en <= 1;  end  // начало импульса en
        VALUE_33:  begin en <= 0;  end	 // конец импульса en	
		 
        VALUE_34:  begin rsn <= 1; data <= "V";   end  
        VALUE_35:  begin en <= 1;  end  // начало импульса en
        VALUE_36:  begin en <= 0;  end	 // конец импульса en	
	
        VALUE_37:  begin rsn <= 1; data <= ver_H;   end  
        VALUE_38:  begin en <= 1;  end  // начало импульса en
        VALUE_39:  begin en <= 0;  end	 // конец импульса en	
		  
        VALUE_40:  begin rsn <= 1; data <= ".";   end  
        VALUE_41:  begin en <= 1;  end  // начало импульса en
        VALUE_42:  begin en <= 0;  end	 // конец импульса en			  

        VALUE_43:  begin rsn <= 1; data <= ver_L;   end  
        VALUE_44:  begin en <= 1;  end  // начало импульса en
        VALUE_45:  begin en <= 0;  end	 // конец импульса en			  
						 			 
        VALUE_1000:
          begin 
		 	  counter_s <= 0; 
			  Init_LCD_start <= 0;  // запретить начальную  инициализацию LCD
			  Funk_LCD_start <= 1;  // разрешить текущее функционирование LCD  
	       end 	  
       endcase
	    //--------------------------------------------
  end

  else if (Funk_LCD_start == 1) // если текущее функционирование LCD разрешено
     begin    
       counter_s <= counter_s + 1;
       //   текущее функционирование LCD - вывод  данных на индикацию 
       case(counter_s) 
		 // задание начального  положения курсора - начало первой строки 
       VALUE_1:  begin rsn <= 0; data <= Init_LCD_1_start;  end  
       VALUE_2:  begin en <= 1;  end  // начало импульса en
       VALUE_3:  begin en <= 0;  end	  
	  
       VALUE_4:  begin rsn <= 1; 
		                  case (mode) 
					    0: data <= "F";
						 1: data <= "f";
				  	   endcase   
                 end  			 
       VALUE_5:  begin en <= 1;  end  // начало импульса en
       VALUE_6:  begin en <= 0;  end	 // конец импульса en 

       VALUE_7:  begin rsn <= 1; data <= "=";   end  
       VALUE_8:  begin en <= 1;  end  // начало импульса en
       VALUE_9:  begin en <= 0;  end	 // конец импульса en 		 
	  	  
       VALUE_10:  begin rsn <= 1; data <= data_fx_in_smill + 48; end  				 
       VALUE_11:  begin en <= 1;  end  // начало импульса en
       VALUE_12:  begin en <= 0;  end	 // конец импульса en 	  

       VALUE_13:  begin rsn <= 1; data <= data_fx_in_dmill + 48; end  			 
       VALUE_14:  begin en <= 1;  end  // начало импульса en
       VALUE_15:  begin en <= 0;  end	 // конец импульса en	  
	  
       VALUE_16:  begin rsn <= 1; data <= data_fx_in_mill + 48; end  			
       VALUE_17:  begin en <= 1;  end  // начало импульса en
       VALUE_18:  begin en <= 0;  end	 // конец импульса en	
		 
       VALUE_19:  begin rsn <= 1; data <= data_fx_in_stis + 48; end  		 
       VALUE_20:  begin en <= 1;  end  // начало импульса en
       VALUE_21:  begin en <= 0;  end	 // конец импульса en	
		 
       VALUE_22:  begin rsn <= 1; data <= data_fx_in_dtis + 48; end  		 
       VALUE_23:  begin en <= 1;  end  // начало импульса en
       VALUE_24:  begin en <= 0;  end	 // конец импульса en	

       VALUE_25:  begin rsn <= 1; data <= data_fx_in_tis + 48; end  
       VALUE_26:  begin en <= 1;  end  // начало импульса en
       VALUE_27:  begin en <= 0;  end	 // конец импульса en

       VALUE_28:  begin rsn <= 1; data <= data_fx_in_sot + 48; end 
		 VALUE_29:  begin en <= 1;  end  // начало импульса en
       VALUE_30:  begin en <= 0;  end	 // конец импульса en		 
			
       VALUE_31:  begin rsn <= 1; data <= data_fx_in_des + 48; end 						
       VALUE_32:  begin en <= 1;  end  // начало импульса en
       VALUE_33:  begin en <= 0;  end	 // конец импульса en	

       VALUE_34:  begin rsn <= 1; data <= data_fx_in_ed + 48; end 								
       VALUE_35:  begin en <= 1;  end  // начало импульса en
       VALUE_36:  begin en <= 0;  end	 // конец импульса en	

       VALUE_37:  begin rsn <= 1; data <= " "; end  
       VALUE_38:  begin en <= 1;  end  // начало импульса en
       VALUE_39:  begin en <= 0;  end	 // конец импульса en	

       VALUE_40:  begin rsn <= 1; data <= "H"; end 						 
       VALUE_41:  begin en <= 1;  end  // начало импульса en
       VALUE_42:  begin en <= 0;  end	 // конец импульса en	

       VALUE_43:  begin rsn <= 1; data <= "z"; end 			 
       VALUE_44:  begin en <= 1;  end  // начало импульса en
       VALUE_45:  begin en <= 0;  end	 // конец импульса en		
				
       VALUE_46:  begin rsn <= 1; data <= " "; end 						 
       VALUE_47:  begin en <= 1;  end  // начало импульса en
       VALUE_48:  begin en <= 0;  end	 // конец импульса en	

       VALUE_49:  begin rsn <= 1; 
		                  case (mode) 
						 0: data <= "I";
						 1: data <= "K";
			 		   endcase   
                  end  				 
       VALUE_50:  begin en <= 1;  end  // начало импульса en
       VALUE_51:  begin en <= 0;  end	 // конец импульса en						
						
		 // задание начального  положения курсора - начало второй строки 
       VALUE_52:  begin rsn <= 0; data <= Init_LCD_2_start;  end  
       VALUE_53:  begin en <= 1;  end  // начало импульса en
       VALUE_54:  begin en <= 0;  end	
		 
       VALUE_55:  begin rsn <= 1; data <= "T";   end  
       VALUE_56:  begin en <= 1;  end  // начало импульса en
       VALUE_57:  begin en <= 0;  end	 // конец импульса en 		 
		 
       VALUE_58:  begin rsn <= 1; data <= "=";   end  
       VALUE_59:  begin en <= 1;  end  // начало импульса en
       VALUE_60:  begin en <= 0;  end	 // конец импульса en 		 
	  	  
       VALUE_61:  begin rsn <= 1; data <= data_px_in_smill + 48; end  				 
       VALUE_62:  begin en <= 1;  end  // начало импульса en
       VALUE_63:  begin en <= 0;  end	 // конец импульса en 	  

       VALUE_64:  begin rsn <= 1; data <= data_px_in_dmill + 48; end  			 
       VALUE_65:  begin en <= 1;  end  // начало импульса en
       VALUE_66:  begin en <= 0;  end	 // конец импульса en	  
	  
       VALUE_67:  begin rsn <= 1; data <= data_px_in_mill + 48; end  			
       VALUE_68:  begin en <= 1;  end  // начало импульса en
       VALUE_69:  begin en <= 0;  end	 // конец импульса en	
		 
       VALUE_70:  begin rsn <= 1; data <= data_px_in_stis + 48; end  		 
       VALUE_71:  begin en <= 1;  end  // начало импульса en
       VALUE_72:  begin en <= 0;  end	 // конец импульса en	
		 
       VALUE_73:  begin rsn <= 1; data <= data_px_in_dtis + 48; end  		 
       VALUE_74:  begin en <= 1;  end  // начало импульса en
       VALUE_75:  begin en <= 0;  end	 // конец импульса en	

       VALUE_76:  begin rsn <= 1; data <= data_px_in_tis + 48; end  
       VALUE_77:  begin en <= 1;  end  // начало импульса en
       VALUE_78:  begin en <= 0;  end	 // конец импульса en

       VALUE_79:  begin rsn <= 1; data <= data_px_in_sot + 48; end 
		 VALUE_80:  begin en <= 1;  end  // начало импульса en
       VALUE_81:  begin en <= 0;  end	 // конец импульса en		 
			
       VALUE_82:  begin rsn <= 1; data <= data_px_in_des + 48; end 						
       VALUE_83:  begin en <= 1;  end  // начало импульса en
       VALUE_84:  begin en <= 0;  end	 // конец импульса en	

       VALUE_85:  begin rsn <= 1; data <= data_px_in_ed + 48; end 								
       VALUE_86:  begin en <= 1;  end  // начало импульса en
       VALUE_87:  begin en <= 0;  end	 // конец импульса en	

       VALUE_88:  begin rsn <= 1; data <= " "; end  
       VALUE_89:  begin en <= 1;  end  // начало импульса en
       VALUE_90:  begin en <= 0;  end	 // конец импульса en	

       VALUE_91:  begin rsn <= 1; data <= "u"; end 						 
       VALUE_92:  begin en <= 1;  end  // начало импульса en
       VALUE_93:  begin en <= 0;  end	 // конец импульса en	

       VALUE_94:  begin rsn <= 1; data <= "s"; end 			 
       VALUE_95:  begin en <= 1;  end  // начало импульса en
       VALUE_96:  begin en <= 0;  end	 // конец импульса en	
 
				
       VALUE_300: begin counter_s <= 0; end 	  
       endcase
	   //--------------------------------------------
  end
end 


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   Контроллер кнопок управления 
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//опрос  кнопок управления    
always @(posedge clk_1_kHz or negedge nrst)          
begin 
 if (!nrst)   
     begin
		 time_strob_reg <= TIME_STROB;  // задать начальное значение длительности строба  
	  end
	  
 else if (edge_noise_key[0] == 0)    
           begin 
				en_buzzer_key[0] <= 1;  // сформировать единичный импульс разрешения звука от кнопки	 
		        //----------------------------------------------------------------------- 
				      time_strob_reg <= time_strob_reg + DELTA_STROB;  // изменение длительности строба  
			     //------------------------------------------------------------------------	    
	        end 
 else if (edge_noise_key[1] == 0)    
            begin 
				 en_buzzer_key[1] <= 1;  // сформировать единичный импульс разрешения звука от кнопки	 
		        //----------------------------------------------------------------------- 
				      time_strob_reg <= time_strob_reg - DELTA_STROB;  // изменение длительности строба  	  
				  //------------------------------------------------------------------------	    
	         end
 else if (edge_noise_key[4] == 0)    
            begin 
				 en_buzzer_key[4] <= 1;  // сформировать единичный импульс разрешения звука от кнопки	 
		        //----------------------------------------------------------------------- 
				 mode <= ~mode;     
   		        //------------------------------------------------------------------------	    
	         end				
 else 
    begin
		  en_buzzer_key[0] <= 0;  // не формировать единичный импульс разрешения звука от кнопки	 
		  en_buzzer_key[1] <= 0;  // не формировать единичный импульс разрешения звука от кнопки
		  en_buzzer_key[4] <= 0;  // не формировать единичный импульс разрешения звука от кнопки		
		  //---------------------------------
	 end	  
end 

// Формирование сигналов LCD  
//------------------------------------------------------------ 
assign LCD_N = 0;
assign LCD_P = 1;
assign dat = data; 
assign rs = rsn;  
assign e = en; 
assign rw = 0;
//-----------------------------------------------------------

// формирование сигналов управления светодиодами
assign led = led_reg; 

// назначение,  какие источники управляют выходом звука:
assign buzzer = 
   ((clk_1_kHz & en_buzzer_button[0]) |  // сигнал разрешения звука от кнопки
	(clk_1_kHz & en_buzzer_button[1]) |  // сигнал разрешения звука от кнопки
   (clk_1_kHz & en_buzzer_button[4]));  // сигнал разрешения звука от кнопки	 
	

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   Контроллер тестовых генераторов  
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

// генератор 125 МГц
always @(posedge clk_pll)	 
begin
  out_125_MHz <= ~out_125_MHz;
end

// генератор 100 Гц
always @(posedge clk)          
begin 
   //------------------------------------
	if (counter_200_Hz == DEL_200_Hz)
	  begin
	     counter_200_Hz <= 0;
        out_100_Hz <= ~out_100_Hz; //
	  end
   else
     begin
        counter_200_Hz <= counter_200_Hz + 1;  	 
	  end
	//------------------------------------- 	
end 

// генератор 1 Гц
always @(posedge clk)          
begin 
   //------------------------------------
	if (counter_2_Hz == DEL_2_Hz)
	  begin
	     counter_2_Hz <= 0;
        out_1_Hz <= ~out_1_Hz; //
	  end
   else
     begin
        counter_2_Hz <= counter_2_Hz + 1;  	 
	  end
	//------------------------------------- 	
end 

// генератор 0,5 Гц
always @(posedge out_1_Hz)	 
begin
  out_05_Hz <= ~out_05_Hz;
end

// генератор 0,25 Гц
always @(posedge out_05_Hz)	 
begin
  out_025_Hz <= ~out_025_Hz;
end

// генератор 0,125 Гц
always @(posedge out_025_Hz)	 
begin
  out_0125_Hz <= ~out_0125_Hz;
end

// генератор 0,00125 Гц
always @(posedge clk)           
begin 
   //------------------------------------
	if (counter_0025_Hz == DEL_0025_Hz)
	  begin
	     counter_0025_Hz <= 0;
        out_00125_Hz <= ~out_00125_Hz; //
	  end
   else
     begin
        counter_0025_Hz <= counter_0025_Hz + 1;  	 
	  end
	//------------------------------------- 	
end 


// формирование сигналов генераторов
assign generator_250_MHz = clk_pll;

assign generator_125_MHz = out_125_MHz;

assign generator_1_MHz = clk_1_MHz; 

assign generator_10_kHz = out_10_kHz;
assign generator_1_kHz = clk_1_kHz; 
assign generator_100_Hz = out_100_Hz;

assign generator_1_Hz = out_1_Hz;
assign generator_05_Hz = out_05_Hz;
assign generator_025_Hz = out_025_Hz;
assign generator_0125_Hz = out_0125_Hz;
assign generator_00125_Hz = out_00125_Hz;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   Модули нижних  уровней
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Модуль PLL  
PLL PLL1(
	.inclk0(clk),
	.c0(clk_pll)
	);	
	
// модуль  фильтра дребезга контактов кнопки key[0]
noise_filter_butt   button_key0(
   .clk(clk_1_kHz), 	
	.key_in(key[0]), 
	.edge_noise_key_out(edge_noise_key[0])
	);	

// модуль  фильтра дребезга контактов кнопки key[1]
noise_filter_butt   button_key1(
   .clk(clk_1_kHz),  	
	.key_in(key[1]), 
	.edge_noise_key_out(edge_noise_key[1])
	);			

// модуль  фильтра дребезга контактов кнопки key[4]
noise_filter_butt   button_key4(
   .clk(clk_1_kHz),  	
	.key_in(key[4]), 
	.edge_noise_key_out(edge_noise_key[4])
	);				
						
// модуль формирования одного импульса (разрешения звука) по нажатию кнопки
buzzer_butt  butt_key0(
   .clk(clk_1_kHz),	
	.en_buzzer_key(en_buzzer_key[0]),
	.en_buzzer_button(en_buzzer_button[0])
	);	
	
// модуль формирования одного импульса (разрешения звука) по нажатию кнопки
buzzer_butt  butt_key1(
   .clk(clk_1_kHz),	
	.en_buzzer_key(en_buzzer_key[1]),
	.en_buzzer_button(en_buzzer_button[1])
	);		
	
// модуль формирования одного импульса (разрешения звука) по нажатию кнопки
buzzer_butt  butt_key4(
   .clk(clk_1_kHz),	
	.en_buzzer_key(en_buzzer_key[4]),
	.en_buzzer_button(en_buzzer_button[4])
	);		
	
endmodule

